@@ -19,15 +19,16 @@ package hostpathprovisioner
 import (
 	"encoding/json"
 	"fmt"
+	"hash/fnv"
 	"os"
 	"reflect"
 
 	jsondiff "github.com/appscode/jsonpatch"
 	jsonpatch "github.com/evanphx/json-patch"
 	"github.com/go-logr/logr"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/jsonmergepatch"
 	"k8s.io/apimachinery/pkg/util/mergepatch"
 )
@@ -58,8 +59,7 @@ func mergeLabelsAndAnnotations(src, dest metav1.Object) {
 	}
 }
 
-func mergeObject(desiredObj, currentObj runtime.Object) (runtime.Object, error) {
-	desiredObj = desiredObj.DeepCopyObject()
+func mergeObject(desiredObj, currentObj client.Object) (client.Object, error) {
 	desiredMetaObj := desiredObj.(metav1.Object)
 	currentMetaObj := currentObj.(metav1.Object)
 
@@ -115,9 +115,9 @@ func logJSONDiff(logger logr.Logger, objA, objB interface{}) {
 	logger.Info("DIFF", "obj", objA, "patch", string(pBytes))
 }
 
-func newDefaultInstance(obj runtime.Object) runtime.Object {
+func newDefaultInstance(obj client.Object) client.Object {
 	typ := reflect.ValueOf(obj).Elem().Type()
-	return reflect.New(typ).Interface().(runtime.Object)
+	return reflect.New(typ).Interface().(client.Object)
 }
 
 func setLastAppliedConfiguration(obj metav1.Object) error {
@@ -154,3 +154,53 @@ func getRecommendedLabels() map[string]string {
 
 	return labels
 }
+
+func getResourceNameWithMaxLength(base, suffix string, maxLength int) string {
+	if maxLength <= 0 {
+		return ""
+	}
+	name := fmt.Sprintf("%s-%s", base, suffix)
+	if len(name) <= maxLength {
+		return name
+	}
+
+	baseLength := maxLength - 10 /*length of -hash-*/ - len(suffix)
+
+	// if the suffix is too long, ignore it
+	if baseLength < 0 {
+		prefix := base[0:min(len(base), max(0, maxLength-9))]
+		// Calculate hash on initial base-suffix string
+		shortName := fmt.Sprintf("%s-%s", prefix, hash(name))
+		return shortName[:min(maxLength, len(shortName))]
+	}
+
+	prefix := base[0:baseLength]
+	// Calculate hash on initial base-suffix string
+	return fmt.Sprintf("%s-%s-%s", prefix, hash(base), suffix)
+}
+
+// hash calculates the hexadecimal representation (8-chars)
+// of the hash of the passed in string using the FNV-a algorithm
+func hash(s string) string {
+	hash := fnv.New32a()
+	hash.Write([]byte(s))
+	intHash := hash.Sum32()
+	result := fmt.Sprintf("%08x", intHash)
+	return result
+}
+
+// max returns the greater of its 2 inputs
+func max(a, b int) int {
+	if b > a {
+		return b
+	}
+	return a
+}
+
+// min returns the lesser of its 2 inputs
+func min(a, b int) int {
+	if b < a {
+		return b
+	}
+	return a
+}
